/**
 * PhysicsElement es una clase abstracta que
 * se usa como padre de cualquier elemento
 * fisico usado en el simulador.
 */
public abstract class PhysicsElement {

	/** Contador de la cantidad de instancias creadas. */
	private static int instCount = 0;
	
	/** Identificador del elemento. */
	private final int myId;
	
	/** Constructor. */
	protected PhysicsElement() {
		/* 
		 * asigna un id y luego incrementa el
		 * contador de instancias.
		 */
		myId = instCount++;
	}
	
	/*
         * Retorna el identificador.
         */
	protected int getId() {
		return myId;
	}
	
	/*
         * Metodos que deben soportar todos los hijos.
         */
	public abstract Vector2D getPosition();
	public abstract float getMass();
	public abstract float getRadius();
	public abstract Vector2D getSpeed();
	public abstract boolean collide(PhysicsElement pe);
	public abstract void computeNextState(double delta_t, MyWorld w);
	public abstract void updateState();
	public abstract String getDescription();
	public abstract String getState();
}
