import java.lang.*;

/**
 * Implements a vector in R2
 */
public class Vector2D {
	private float x;
	private float y;
	
	/**
	 * Constructor of the vector with x and y as parameters.
	 */
	public Vector2D(float x, float y) {
            setX (x);
            setY (y);
	}
	
	/**
	 * Constructor, creates a new vector with the same coordinates as an other vector.
	 */
	public Vector2D(Vector2D vector) {
            setX (vector.getX ());
            setY (vector.getY ());
	}
	
	/**
	 * Returns a new vector with coordinates -x and -y
	 */
	public Vector2D getNegative() {
            return new Vector2D (-this.x, -this.y);
	}
	
	/**
	 * Returns a new vector that is perpendicular to this vector.
	 */
	public Vector2D getPerpendicular() {
            return new Vector2D (-this.y, this.x);
	}
	
	/**
	 * Returns a new vector that is the proyection of this vector over the vector passed as parameter.
	 */
	public Vector2D getProjection(Vector2D vector) {
            Vector2D unitVector = vector.getUnitVector ();
            float projectionMagnitude = dot (unitVector);
            return unitVector.getScaled (projectionMagnitude);
	}
	
	/**
	 * Returns the magnitude of the proyection of this vector over the vector passed as parameter.
	 */
	public float getProjectionMagnitude(Vector2D vector) {
            Vector2D unitVector = vector.getUnitVector ();
            return dot (unitVector);
	}
	
	/**
	 * Returns a new vector that is scaled by a factor passed as parameter.
	 */
	public Vector2D getScaled(float factor) {
            Vector2D newVector = new Vector2D(this.x, this.y);
            newVector.scale(factor);
            return newVector;
	}
	
	/**
	 * Returns a new vector that is scaled by a factor passed as parameter.
	 */
	public Vector2D getScaled(double factor) {
            Vector2D newVector = new Vector2D(this.x, this.y);
            newVector.scale(factor);
            return newVector;
	}
	
	/**
	 * Returns a new vector that is the subtraction of this vector with the vector passed as parameter.
	 */
	public Vector2D getSubtraction(Vector2D subtrahend) {
            Vector2D newVector = new Vector2D (this.x, this.y);
            newVector.subtact (subtrahend);
            return newVector;
	}
	
	/**
	 * Returns a new vector that is the addition of this vector and a vector passed as parameter.
	 */
	public Vector2D getSum(Vector2D sumVector) {
            Vector2D newVector = new Vector2D (this.x, this.y);
            newVector.sum (sumVector);
            return newVector;
	}

	/**
	 * Returns a new vector that is the unitary vector of this vector.
	 */
	public Vector2D getUnitVector() {
            Vector2D newVector;
            newVector = new Vector2D (this.x/this.norm(), this.y/this.norm());
            return newVector;
	}
	
	/**
	 * Returns the X property.
	 */
	public float getX() {
            return this.x;
	}
	
	/**
	 * Returns the Y property.
	 */
	public float getY() {
            return this.y;
	}
	
	/**
	 * Returns the dot product of this vector and the vector passed as parameter.
	 */
	public float dot(Vector2D vector) {
            return this.x*vector.getX() + this.y*vector.getY();
	}
	
	/**
	 * Returns the euclidean norm of this vector.
	 */
	public float norm() {
            return (float) Math.sqrt (Math.pow (this.x, 2) + Math.pow (this.y, 2));
	}
	
	/**
	 * Scale this vector by a factor passed as parameter.
	 */
	public Vector2D scale(float factor) {
            this.x = this.x * factor;
            this.y = this.y * factor;
            return this;
	}
	
	/**
	 * Scale this vector by a factor passed as parameter.
	 */
	public Vector2D scale(double factor) {
            this.x = this.x * (float)factor;
            this.y = this.y * (float)factor;
            return this;
	}
	
	/**
	 * Subtract a vector passed as parameter to this vector.
	 */
	public Vector2D subtact(Vector2D subtrahend) {
            sum (subtrahend.getNegative ());
            return this;
	}
	
	/**
	 * Add a vector passed as parameter to this vector.
	 */
	public Vector2D sum(Vector2D sumVector) {
            this.x = this.x + sumVector.getX();
            this.y = this.y + sumVector.getY();
            return this;
	}
	
	/**
	 * Set the properties X and Y of this vector.
	 */
	public void setVector(float x, float y) {
            setX(x);
            setY(y);
	}
	
	/**
	 * Set the property X of this vector.
	 */
	public void setX(float x) {
            this.x = x;
	}
	
	/**
	 * Set the property Y of this vector.
	 */
	public void setY(float y) {
            this.y = y;
	}
}
